package guest;

import java.io.Serializable;
import java.sql.Date;
import java.util.Objects;
import java.text.Collator;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Basic;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Cacheable;

@Entity
@Cacheable
@NamedQueries( {
@NamedQuery(
		name="Guest.findByName",
		query="select g from guest.Guest g where lower( g.name ) = :name"
),
	@NamedQuery(
		name="Guest.findAll",
		query="select g from guest.Guest g order by g.name"
) } )

public class Guest implements Serializable, Comparable<Guest> {
    private static final long serialVersionUID = 1L;

    // Persistent Fields:
    @Id @GeneratedValue
    Long id;
	
	@Basic
    private String name;
	
	@Basic
    private Date signingDate;

    // Constructors:
    public Guest() {
    }

    public Guest(String name) {
        this.setName(name);
        this.signingDate = new Date(System.currentTimeMillis());
    }


	
	public Long getId() {
		return id;
	}
	
	public String getName() {
		return name;
	}
	
	public void setName( String name ) {
		this.name = name != null ? name.toLowerCase() : null;
	}
	
	@Override public String toString() {
        return " " + this.getId() + ":" + this.getName() + ":" + signingDate ;
    }
	
	@Override public int hashCode() {
		return Objects.hashCode( this.getName() );
		// return Objects.hashCode( this.getId() );
	}
	
	@Override public boolean equals(Object obj) {
		if ( obj == this ) return true;
		if ( obj == null ) return false;
		boolean eq = false;
		if ( obj instanceof Guest ) {
			final Guest other = Guest.class.cast( obj );
			eq = this.compareTo( other ) == 0;
		}
		return eq;
	}
	
	@Override public int compareTo( final Guest other ) {
		if( other == null )  return 1;
		if ( this.getName() == null && other.getName() == null ) return 0;
		if ( this.getName() == null  ) return -1;
		if(  other.getName() == null ) return 1;

		final Collator collator = Collator.getInstance();
		collator.setStrength( Collator.SECONDARY );
		int result = collator.compare( this.getName(), other.getName() );
		return result;
	}
}