package guest;
 
import java.util.List;
import javax.ejb.Stateless;
import javax.ejb.Stateful;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;

import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;
import javax.persistence.PersistenceUnit;

import javax.persistence.TypedQuery;
import javax.persistence.metamodel.ManagedType;

import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;

	
@Stateless
@TransactionManagement(TransactionManagementType.CONTAINER)
@TransactionAttribute(TransactionAttributeType.REQUIRED)
public class GuestDao {

	@PersistenceUnit(unitName = "guest-pu" ) private EntityManagerFactory emf;
	// not working with JBOSS AS 7.1.1.Final unless type of PersistenceContext is EXTENDED and SessionBean becomes @Stateful instead of @Stateless
	// @PersistenceContext(unitName = "guest-pu" ) private EntityManager em;

    // Stores a new guest
    public void persist(final Guest guest) {
		final EntityManager em = getEntityManager();
		Guest persistedGuest = findByName( guest.getName() );
		if ( persistedGuest == null ) {
			em.persist(guest);
		}
    }
	
    // Retrieves all the guests:
	public List<Guest> findAll() {
		final EntityManager em = getEntityManager();
		final TypedQuery<Guest> query = em.createNamedQuery( "Guest.findAll", Guest.class );
		final List<Guest> guests =  query.getResultList();
		return guests;
    }
	
	public Guest findByName( final String name ) {
		final EntityManager em = getEntityManager();
		Guest guest = null;
		if ( name != null ) {
			TypedQuery<Guest> query = em.createNamedQuery( "Guest.findByName", Guest.class )
				.setParameter( "name", name.toLowerCase() );
			final List<Guest> guests =  query.getResultList();
			if ( guests.size() > 0 ) guest = guests.get(0);
		}
		return guest;
    }
	
	public EntityManager getEntityManager() {
		return this.emf.createEntityManager();
	}
}